@@ -184,6 +184,8 @@ typedef enum _MpdSearchType {
 	MPD_SEARCH_TYPE_NONE,
 	MPD_SEARCH_TYPE_FIND,
 	MPD_SEARCH_TYPE_SEARCH,
-	MPD_SEARCH_TYPE_LIST
+	MPD_SEARCH_TYPE_LIST,
+	MPD_SEARCH_TYPE_PLAYLIST_FIND,
+	MPD_SEARCH_TYPE_PLAYLIST_SEARCH
 }MpdSearchType;
 #endif
