@@ -791,8 +791,8 @@ MpdData* mpd_data_concatenate( MpdData  * const first, MpdData  * const second)
 {
 	MpdData_real *first_real  = (MpdData_real*)first;
 	MpdData_real *second_real = (MpdData_real*)second;
-	MpdData_head *first_head  = mpd_data_get_head(first);
-	MpdData_head *second_head = mpd_data_get_head(second);
+	MpdData_head *first_head  = NULL;
+	MpdData_head *second_head = NULL; 
 	MpdDataPool *pool;
 
 	if ( first == NULL ) {
@@ -805,6 +805,9 @@ MpdData* mpd_data_concatenate( MpdData  * const first, MpdData  * const second)
 			return (MpdData*)first_real;
 	}
 
+	first_head = mpd_data_get_head(first);
+	second_head = mpd_data_get_head(second);
+
 	/* find last element in first data list */	
 	while (!mpd_data_is_last((MpdData*)first_real)) first_real = (MpdData_real*)mpd_data_get_next_real((MpdData*)first_real, FALSE);
 	second_real =(MpdData_real*) mpd_data_get_first((MpdData*)second_real);
@@ -912,7 +915,7 @@ static void mpd_free_queue_ob(MpdObj *mi)
 	}
 	if(mi->queue == NULL)
 	{
-		debug_printf(DEBUG_INFO, "mi->queue != NULL failed");
+		debug_printf(DEBUG_INFO, "mi->queue != NULL failed, nothing to clean.");
 		return;
 	}	
 	mi->queue = mi->queue->first;
