@@ -17,6 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * \example testcase.c
+ * A small example of a console client using libmpd.
+ */
+
 /** \defgroup 1Basic Basic
  */
 /*@{*/
@@ -44,6 +49,8 @@ extern "C" {
 /** Defined for readability: False is 0. */
 #define FALSE 0
 #endif
+#include "libmpd-version.h"
+extern char *libmpd_version;
 
 /**
  * Enum that represent the errors libmpd functions can return 
@@ -200,6 +207,13 @@ MpdObj *mpd_new(char *hostname, int port, char *password);
  */
 int mpd_set_hostname(MpdObj * mi, char *hostname);
 
+/** 
+ * @param mi a #MpdObj
+ *
+ * gets the set hostname 
+ * returns: a const char
+ */
+const char * mpd_get_hostname(MpdObj *mi);
 	
 /**
  * @param mi a #MpdObj
@@ -551,7 +565,13 @@ long unsigned mpd_server_get_database_update_time(MpdObj * mi);
  */
 int mpd_server_check_version(MpdObj * mi, int major, int minor, int micro);
 
+/**
+ * @param mi a #MpdObj
+ *
+ * @return a string with version or NULL when not connected
+ */
 
+char *mpd_server_get_version(MpdObj *mi);
 /**
  * @param mi a #MpdObj
  * @param command the command to check
