@@ -1,5 +1,5 @@
 /* libmpdclient
-   (c)2003-2007 by Warren Dukes (warren.dukes@gmail.com)
+   (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
    This project's homepage is: http://www.musicpd.org
 
    Redistribution and use in source and binary forms, with or without
@@ -227,12 +227,21 @@ typedef struct _mpd_Stats {
 	unsigned long dbPlayTime;
 } mpd_Stats;
 
+typedef struct _mpd_SearchStats {
+	int numberOfSongs;
+	unsigned long playTime;
+} mpd_SearchStats;
+
 void mpd_sendStatsCommand(mpd_Connection * connection);
 
 mpd_Stats * mpd_getStats(mpd_Connection * connection);
 
 void mpd_freeStats(mpd_Stats * stats);
 
+mpd_SearchStats * mpd_getSearchStats(mpd_Connection * connection);
+
+void mpd_freeSearchStats(mpd_SearchStats * stats);
+
 /* SONG STUFF */
 
 #define MPD_SONG_NO_TIME	-1
@@ -418,10 +427,10 @@ void mpd_sendListallInfoCommand(mpd_Connection * connection, const char * dir);
 /* non-recursive version of ListallInfo */
 void mpd_sendLsInfoCommand(mpd_Connection * connection, const char * dir);
 
-#define MPD_TABLE_ARTIST	0
-#define MPD_TABLE_ALBUM		1
-#define MPD_TABLE_TITLE		2
-#define MPD_TABLE_FILENAME	3
+#define MPD_TABLE_ARTIST	MPD_TAG_ITEM_ARTIST
+#define MPD_TABLE_ALBUM		MPD_TAG_ITEM_ALBUM
+#define MPD_TABLE_TITLE		MPD_TAG_ITEM_TITLE
+#define MPD_TABLE_FILENAME	MPD_TAG_ITEM_FILENAME
 
 void mpd_sendSearchCommand(mpd_Connection * connection, int table,
 		const char * str);
@@ -567,6 +576,14 @@ void mpd_sendNotCommandsCommand(mpd_Connection * connection);
  */
 char *mpd_getNextCommand(mpd_Connection *connection);
 
+void mpd_sendUrlHandlersCommand(mpd_Connection * connection);
+
+char *mpd_getNextHandler(mpd_Connection * connection);
+
+void mpd_sendTagTypesCommand(mpd_Connection * connection);
+
+char *mpd_getNextTagType(mpd_Connection * connection);
+
 /**
  * @param connection a MpdConnection
  * @param path	the path to the playlist.
@@ -599,7 +616,7 @@ void mpd_startSearch(mpd_Connection *connection, int exact);
  * @param type
  * @param name
  */
-void mpd_addConstraintSearch(mpd_Connection *connection, int type, char *name);
+void mpd_addConstraintSearch(mpd_Connection *connection, int type, const char *name);
 
 /**
  * @param connection a #mpd_Connection
@@ -628,6 +645,21 @@ void mpd_commitSearch(mpd_Connection *connection);
  * mpd_getNextTag to get the results
  */
 void mpd_startFieldSearch(mpd_Connection *connection, int type);
+
+void mpd_startPlaylistSearch(mpd_Connection *connection, int exact);
+
+void mpd_startStatsSearch(mpd_Connection *connection);
+
+void mpd_sendPlaylistClearCommand(mpd_Connection *connection, char *path);
+
+void mpd_sendPlaylistAddCommand(mpd_Connection *connection,
+                                char *playlist, char *path);
+
+void mpd_sendPlaylistMoveCommand(mpd_Connection *connection,
+                                 char *playlist, int from, int to);
+
+void mpd_sendPlaylistDeleteCommand(mpd_Connection *connection,
+                                   char *playlist, int pos);
 #ifdef __cplusplus
 }
 #endif
